C**********************************************************************
C*
C*            PROGRAM TO COMPUTE WAVE REFRACTION USING
C*               IRROTATIONALITY OF THE WAVE NUMBER
C*
C*               INCLUDES WAVE-CURRENT INTERACTION
C*
C*                     ROBERT A. DALRYMPLE
C*
C*                        January, 1991
C*
C*     BASED ON A TWO STEP LAX-WENDROFF  FINITE DIFFERENCES 
C*                   
C*                       FULLY  EXPLICIT
C*
C**********************************************************************
      PARAMETER(IX=85,IY=85)
      COMMON /AMP/K(2,IY),B(2,IY),A(2,IY),D(IX,IY)
      COMMON U(IX,IY),V(IX,IY),WAX(2,IY),WAY(2,IY)
      COMMON UCGX(2,IY),VCGY(2,IY)
      COMMON /FREQ/OMEGA,SIGMA(IY),CG(IY),NONLIN
      COMMON /WAVE/H0,T,TH(IY),H(IY)
      DIMENSION BS(IY),WYS(IY)
      REAL K
      OPEN(3,FILE='indat.dat')
C*----------------------------------------------------------------------
C*                   DATA FILES                                        *
C*                                                                     *  
C*                                                                     *
C*    refdat.dat --  MUST CONTAIN:                                     *
C*                                                                     *
C*        ((D(I,J),J=1,N),I=1,M)      BATHYMETRIC GRID        (16F8.4) *
C*                                                                     *
C*        IF CURRENTS ARE PRESENT (ICUR = 1) THEN REFDAT.DAT ALSO HAS  *
C*                                                                     *
C*        ((U(I,J),J=1,N),I=1,M)      ONSHORE CURRENT FIELD        "   *
C*        ((V(I,J),J=1,N),I=1,M)      ALONGSHORE CURRENT FIELD     "   *
C*                                                                     *
C*                                                                     *
C*    indat.dat --  MUST CONTAIN:                                      *
C*                                                                     *
C*        IUN1,IUN2,IUN3,IUN4    LOGICAL DEVICE NUMBERS         (15I4) *
C*                                                                     *
C*            IUN1  :  INPUT  DISK FILE, refdat.dat                    *     
C*            IUN2  :  OUTPUT DISK FILE, height.dat -- WAVE HEIGHTS    *     
C*            IUN3  :  SCREEN OUTPUT -- USE ZERO FOR PC                *     
C*            IUN4  :  OUTPUT DISK FILE, angle.dat  -- WAVE ANGLE      *     
C*              (NOTE:  do not use 3 as a number!)                     *
C*                                                                     *
C*        M,N,IDEEP              GRID DIMENSIONS, OFFSHORE DATA SWITCH *
C*                                                                     *
C*            IDEEP = 0 MEANS DEEP WATER WAVE DATA                     *
C*            IDEEP = 1 MEANS DATA FOR OFFSHORE GRID ROW               *
C*                                                                     *
C*        IU,NONLIN,ICUR,IBC     UNITS, MODEL, CURRENTS, BOUNDARY      *
C*                                       TYPE SWITCHES                 *
C*            UNITS  = 1  METRIC         2  ENGLISH SYSTEM             *
C*            NONLIN = 0  LINEAR MODEL   1  NONLINEAR                  *
C*            ICUR   = 0  NO CURRENTS    1  CURRENT CASE               *
C*            IBC    = 0  TRANSMITTING   1  REFLECTING                 *
C*                                                                     *
C*        DX,DY                  GRID SIZE                    (16F8.4) *
C*        T,TIDE                 WAVE PERIOD AND TIDAL OFFSET          *
C*        AMP,THETA0             WAVE AMPLITUDE AND DIRECTION          *
C*                                  (THETA0 BETWEEN -90 TO 90 DEGREES) *
C*                                                                     *
C*      height.dat  --  MUST EXIST ON DEFAULT DRIVE BEFORE RUNNING     * 
C*                                                                     *
C*         ON COMPLETION, IT CONTAINS:                                 *
C*                                                                     *
C*            M,N,DX,DY                                                *
C*            (H(J),J=1,N)  IN 16F8.4 FORMAT                           *
C*                                                                     *
C*      angle.dat  --  MUST EXIST ON DEFAULT DRIVE BEFORE RUNNING      *
C*                                                                     *
C*         ON COMPLETION, IT CONTAINS:                                 *
C*                                                                     *
C*            M,N,DX,DY                                                *
C*            TH(J),J=1,N   IN DEGREES AND IN 16F8.4 FORMAT            *
C*                                                                     *
C*----------------------------------------------------------------------
C*    DATA
      PI=3.1415927
      G=9.81
      ERROR=0.000001
      CONVD=180./PI
C*----------------------------------------------------------------------
C*    READ INDAT.DAT DATA                                              *
C*----------------------------------------------------------------------
      READ(3,*) IUN1,IUN2,IUN3,IUN4 
  33  FORMAT(15I4)
      READ(3,*) M,N,IDEEP
      READ(3,*) IU,NONLIN,ICUR,IBC
      IF(IU.EQ.2) G=32.17    
      READ(3,*) DX,DY
  34  FORMAT(16F8.4)
      READ(3,*) T,TIDE
      OMEGA=2.*PI/T    
      READ(3,*) AMP, THETA0
      IF(ABS(THETA0).GT.90.) THEN
        WRITE(IUN3,121) 
 121    FORMAT('  WAVE ANGLE TOO LARGE--STOPPING')
        STOP
      ENDIF
      CLOSE (3)
      H0=2.*AMP
C*----------------------------------------------------------------------
C*    SET BREAKING SWITCH ON TO ENABLE THE BREAKING PROCEDURES         *
C*----------------------------------------------------------------------
      IBREAK=1
C*----------------------------------------------------------------------
C*    READ DEPTH AND CURRENT DATA  FROM REFDAT.DAT                     *
C*----------------------------------------------------------------------
      OPEN(IUN1,FILE='refdat.dat')
      OPEN(IUN2,FILE='height.dat')
      OPEN(IUN4,FILE='angle.dat')
C*----------------------------------------------------------------------
C*        IF THETA0 NEGATIVE, MIRROR IMAGE THE DEPTH AND CURRENT GRID  *
C*        AND CHANGE THE SIGN FOR THE MIRROR IMAGE DEPTHS              *
C*----------------------------------------------------------------------
      IF (THETA0.LT.0.) THEN
              THETA0=-THETA0
              JF=N
              JL=1
              JS=-1
      ELSE 
              JF=1
              JL=N
              JS=1  
      ENDIF
      DO 2 I=1,M
          READ(IUN1,10) (D(I,J),J=JF,JL,JS)
    2 CONTINUE
      IF (TIDE.NE.0.0) THEN
       DO 53 I=1,M
         DO 53 J=1,N
           D(I,J)=D(I,J)+TIDE
  53   CONTINUE 
      ENDIF
      IF(ICUR.EQ.1) THEN
       DO 3 I=1,M  
           READ(IUN1,10) (U(I,J),J=JF,JL,JS)
    3  CONTINUE
       DO 4 I=1,M
           READ(IUN1,10) (V(I,J),J=JF,JL,JS)
    4  CONTINUE
      ELSE
       DO 13 I=1,M
         DO 13 J=1,N
           U(I,J)=0.0
           V(I,J)=0.0
   13  CONTINUE  
      ENDIF
 10   FORMAT(16F8.4)
      CLOSE(IUN1)
      R=DX/(2.*DY)
      WRITE(IUN2,333) M,N,DX,DY
      WRITE(IUN4,333) M,N,DX,DY
 333  FORMAT(2I14,2F18.7)
      WRITE(IUN3,100) 
 100  FORMAT(/////'                 REFRACT'//'   A refraction model for
     1 the calculation of wave heights'/'       and directions over a gi
     2ven bathymetry.'//)
      WRITE(IUN3,99)
  99  FORMAT('  Developed by: Robert A. Dalrymple'/'                Cent
     1er for Applied Coastal Research'/'                University of De
     2laware'/'                Newark, DE 19716') 
      WRITE(IUN3,98)
  98  FORMAT(/'                Version 2.1 ')        
      WRITE(IUN3,101) H0,T,THETA0
 101  FORMAT(///'  INCIDENT WAVE PARAMETERS:'//'  Height =',F6.2
     1,', Period =',F6.2,', Wave Angle=',F6.2)
      WRITE(IUN3, 107) M,N,DX,DY
 107  FORMAT(/ '  Grid Parameters:  M =',I4,', N=', I4/
     1 '                    DX=',F8.2,', DY=',F8.2)
      IF(NONLIN.EQ.1)THEN
        WRITE(IUN3,103)
 103    FORMAT(/'  Nonlinear Model') 
      ELSE
        WRITE(IUN3,105)
 105    FORMAT(/'  Linear Model')
      ENDIF   
      IF (IBC.EQ.1) THEN
        WRITE(IUN3,104)
 104    FORMAT(/'  Reflecting Downwave Boundary Condition')
      ELSE
        WRITE(IUN3,106)
 106    FORMAT(/'  Transmitting Boundary Condition')
      ENDIF
C*---------------------------------------------------------------------
C*       ESTABLISH DEEP WATER DATA                                    *
C*---------------------------------------------------------------------
      CALL WVNUM(D(1,1),0.,0.,1,1,0.,0.,K(1,1),1,DB,G)
      CG0=CG(1)
      C0=OMEGA/K(1,1)
      THETA0=THETA0/CONVD
C*---------------------------------------------------------------------
C*    IF INPUT DATA ARE DEEP WATER VALUES (IDEEP.EQ.0) THEN CONVERT   *
C*               BY  SNELLS LAW                                       *
C*---------------------------------------------------------------------        
      IF (IDEEP.EQ.0)THEN
C*                         CONVERT TO VALUES AT OFFSHORE GRID BOUNDARY
         CG00=G*T/(4.*PI)

         COS0=COS(THETA0)
         SIN0=SIN(THETA0)
         THETA0=ASIN(SIN0*C0/(2.*CG00))
         H0=H0*SQRT(CG00/CG(1))*SQRT(COS0/COS(THETA0))
      ENDIF
      COS0=COS(THETA0)
      SIN0=SIN(THETA0)
 14   FORMAT(16F8.3)
C*----------------------------------------------------------------------
C*                                                                     *
C*    SET UP BOUNDARY CONDITIONS ON A AND B                            *
C*                                                                     *
C*            OFFSHORE ROW                                             *
C*----------------------------------------------------------------------
      DO 15 J=1,N
        TH(J)=THETA0
        H(J)=H0
        CALL WVNUM(D(1,J),THETA0,H0,1,J,U(1,J),V(1,J),K(1,J),2,DB,G)
        A(1,J)=K(1,J)*SIN(TH(J))
        B(1,J)=K(1,J)*COS(TH(J))
 15   CONTINUE
      WRITE(IUN3,*) '    '
      WRITE(IUN3,*) ' INITIAL THETA VALUES'
      WRITE(IUN3,*) '    '
      WRITE(IUN3,14) (TH(JJ)*CONVD*JS,JJ=JF,JL,JS)
C*----------------------------------------------------------------------
C*    WRITE ANGLES TO FILE, ANGLE.DAT IN RADIANS                       *
C*----------------------------------------------------------------------
      WRITE(IUN4,14) (TH(JJ)*CONVD*JS,JJ=JF,JL,JS)
      WRITE(IUN3,*) '    '
      WRITE(IUN3,*) ' INITIAL HEIGHT VALUES'
      WRITE(IUN3,*) '    '
      WRITE(IUN3,14) (H(JJ),JJ=JF,JL,JS) 
      WRITE(IUN2,14) (H(JJ),JJ=JF,JL,JS)
      AINT=A(1,1)
C*----------------------------------------------------------------------
C*                                                                     *
C*    SET UP OFFSHORE BOUNDARY CONDITION FOR WAVE ACTION               *
C*                                                                     *
C*----------------------------------------------------------------------
      DO 200 J=1,N                  
        EF=H0*H0
        WAX(1,J)=EF*(U(1,J)+CG(1)*COS(THETA0))/SIGMA(J) 
 200    WAY(1,J)=EF*(V(1,J)+CG(1)*SIN(THETA0))/SIGMA(J)
      WAX0=WAX(1,1)
      B(2,1)=B(1,1)
      WRITE(IUN3,*) '    '
C*----------------------------------------------------------------------
C*                                                                     *
C*    START THE SOLUTION TECHNIQUE                                     *
C*                                                                     *
C*----------------------------------------------------------------------
      ITER1=1
      IF (NONLIN.EQ.0) ITER1=2
      DO 500 I=1,M-1
        I1=I+1
        XX=FLOAT(I)*DX
        WRITE(IUN3,999) I1, XX
  999   FORMAT(/'  ROW',I4,', X =',F10.2/)
C*----------------------------------------------------------------------
C*                                                                     *
C*     ESTABLISH UPWAVE BOUNDARY--SNELL'S LAW                          *
C*---------------------------------------------------------------------- 
        A(2,1)=AINT
        WAX(2,1)=WAX0
        DO 22 IT=1,ITER1
          H(1)=SQRT(ABS(WAX(2,1)*SIGMA(1)/(CG(1)*COS(TH(1))+U(I1,1))))
c*-------------------------------------------------------------------
c*        INDICATE PRESENCE OF DRY LAND ON BOUNDARY
c*-------------------------------------------------------------------      
          IF(D(I1,1).LT.0.) THEN
            WRITE(IUN3,*) ' DRY LAND: I=',I1
            D(I1,1)=0.01
            K(2,1)=2.*PI/(SQRT(G*D(I1,1))*T)
          ELSE
            CALL WVNUM(D(I1,1),TH(1),H(1),I1,1,U(I1,1),
     1               V(I1,1),K(2,1),1,AINT,G)
          ENDIF
          BFIX=SQRT(K(2,1)**2.-AINT**2.)
          TH(1)=ATAN2(AINT,BFIX)
          UCGX(2,1)=U(I1,1)+CG(1)*COS(TH(1))
          VCGY(2,1)=V(I1,1)+CG(1)*SIN(TH(1))
          EB=(.8*D(I1,1))**2
          WAXB=EB*UCGX(2,1)/SIGMA(1)
C*-------------------------------------------------------------------
C*    DETERMINE START OF BREAKING ALONG UPWAVE BOUNDARY
C*-------------------------------------------------------------------
          IF (WAXB.LT.ABS(WAX0).AND.IBREAK.EQ.1) THEN
            WAX(2,1)=WAXB
            H(1)=.8*D(I1,1)
            WAY(2,1)=EB*VCGY(2,1)/SIGMA(1)
          ELSE
            WAX(2,1)=WAX0
            WAY(2,1)=WAX0*VCGY(2,1)/UCGX(2,1)
          END IF 
          B(2,1)=BFIX
   22   CONTINUE
        B(2,1)=BFIX
        THT=TH(1)
C*----------------------------------------------------------------------
C*    ITERATE FOR J TH SOLUTION (ITERATIONS ARE NEEDED FOR THE LINEAR  *
C*      MODEL WITH CURRENTS AND THE NONLINEAR MODEL)                   *

C*----------------------------------------------------------------------
      ITER2=ITER1
      IF(ICUR.EQ.0.AND.NONLIN.EQ.0) ITER2=1
       DO 70 ITERA=1,ITER2
         DO 60 J=1,N-1
C*----------------------------------------------------------------------
C*       FOR EACH COLUMN, OBTAIN A NEW WAVE NUMBER AT I+1,J+1          *
C*----------------------------------------------------------------------
         L=J+1
         CALL WVNUM(D(I1,L),THT,H(L),I1,L,U(I1,L),V(I1,L),K(2,L),2,DB,G)
C*----------------------------------------------------------------------
C*       KEEP TRACK OF ACTUAL COLUMN NUMBER (FOR CASE OF FLIPPED GRID) *
C*---------------------------------------------------------------------- 
         IF(JF.EQ.N) THEN 
            JJJ=N-J+1
         ELSE 
            JJJ=J
         ENDIF 
C*----------------------------------------------------------------------
C*       FIRST LAX HALF STEP:  COMPUTE A(i+1/2,j+1/2)                  *
C*----------------------------------------------------------------------
          AS=(A(1,J+1)+A(1,J))/2. + R*(B(1,J+1)-B(1,J))
          XKB=(K(1,J)+K(1,J+1)+K(2,J)+K(2,J+1))/4.
          ARG=XKB*XKB-AS*AS
          IF(ARG.LT.0.0) THEN
               ARG=0.0
               WRITE(*,*) ' ERROR IN A (1ST LAX STEP) AT',JJJ
          ENDIF
	  BS(J)=SQRT(ARG)
   60    CONTINUE
C*----------------------------------------------------------------------
C*       SET BOUNDARY CONDITION                                        *
C*----------------------------------------------------------------------
         XKB=(K(1,N)+K(2,N))/2.
         IF(IBC.EQ.0) THEN
C*                         TRANSMITTING BC
 	  AS=A(1,N)
          ARG=XKB**2-AS**2
          IF(ARG.LT.0.0) THEN
             ARG=0.0
             WRITE(*,*) ' ERROR IN BOUNDARY VALUE OF A AT',JJJ
          ENDIF
          BS(N)=SQRT(ARG)
         ELSE
          BS(N)=XKB
         ENDIF
C*----------------------------------------------------------------------
C*                                                                     *
C*       SECOND LAX  STEP                                              *
C*                                                                     *
C*----------------------------------------------------------------------
         DO 61 J=2,N 
          A(2,J)=A(1,J)+2.*R*(BS(J)-BS(J-1))
	  ARG=(K(2,J)**2-A(2,J)**2)
C*----------------------------------------------------------------------
C*        KEEP TRACK OF ACTUAL COLUMN NUMBER IN JJJ                    *
C*----------------------------------------------------------------------
          IF(JF.EQ.N) THEN 
              JJJ=N-J+1
          ELSE 
              JJJ=J
          ENDIF
C*----------------------------------------------------------------------
C*        CHECK FOR ERRORS
C*----------------------------------------------------------------------
          IF(ABS(A(2,J)).GT.K(2,J)) THEN
               A(2,J)=(.98*K(2,J))*A(2,J)/ABS(A(2,J))
               WRITE(*,*) ' ERROR IN A (2ND LAX STEP) AT', JJJ
               ARG=.03*K(2,J)**2
          ENDIF
               B(2,J)=SQRT(ARG)
               TH(J)=ASIN(A(2,J)/K(2,J))
   61    CONTINUE
C*----------------------------------------------------------------------
C*                                                                     *
C*         SOLVE FOR WAVE ACTION, WAX(I+1,J) USING FINITE DIFFERENCES  *
C*                                                                     *
C*               FIRST LAX HALF STEP                                   *
C*                                                                     *
C*----------------------------------------------------------------------
        DO 62 J=1,N-1
          WXS=(WAX(1,J+1)+WAX(1,J))/2. - R*(WAY(1,J+1)-WAY(1,J))
          L=J+1
          UCGX(2,L)=(U(I1,L)+CG(L)*COS(TH(L)))
          VCGY(2,L)=(V(I1,L)+CG(L)*SIN(TH(L)))
          VUR=(VCGY(1,J)+VCGY(2,J)+VCGY(1,L)+VCGY(2,L))
     1         /(UCGX(1,J)+UCGX(2,J)+UCGX(1,L)+UCGX(2,L))
          WYS(J)=WXS*VUR
C       BE SURE TO CHANGE DEFINITION FOR VCGY FOR HALF STEP
   62   CONTINUE
        UCGX(2,N)=(U(I1,N)+CG(N)*COS(TH(N)))
        VCGY(2,N)=(V(I1,N)+CG(N)*SIN(TH(N)))
	VUR=(VCGY(1,N)+VCGY(2,N))/(UCGX(1,N)+UCGX(2,N))
C*----------------------------------------------------------------------
C*      SET BOUNDARY CONDITION                                         *
C*----------------------------------------------------------------------
	IF(IBC.EQ.0) THEN
	  WXS=WAX(1,N)
	  WYS(N)=WXS*VUR
	ELSE
          WXS=WAX(1,N)+R*2.*WAY(1,N) 
          WYS(N)=WXS*VUR
	ENDIF
C*----------------------------------------------------------------------
C*                                                                     *
C*            SECOND LAX  STEP                                         *
C*                                                                     *
C*----------------------------------------------------------------------
        DO 63 J=2,N
         WAX(2,J)=WAX(1,J)-2.*R*(WYS(J)-WYS(J-1))
         WAYFIX=WAX(2,J)*VCGY(2,J)/UCGX(2,J) 
C*----------------------------------------------------------------------
C*         CHECK FOR BREAKING                                          *
C*----------------------------------------------------------------------
            IF (IBREAK.EQ.0) GO TO 77
             EB=(.8*D(I1,J))**2
             WAXB=EB*UCGX(2,J)/SIGMA(J)
             IF (WAXB.LT.ABS(WAX(2,J))) THEN
C                        BREAKING OCCURS
               WAX(2,J)=WAXB
                      IF(IBC.EQ.0)THEN  
              WAYFIX=EB*VCGY(2,J)/SIGMA(J)
             ENDIF 
             IF(JF.EQ.N) THEN 
                   JJJ=N-J+1
               ELSE 
                   JJJ=J
             ENDIF
             WRITE(*,*) ' BREAKING, COLUMN',JJJ, ', DEPTH',   D(I1,J)
             ENDIF
   77    CONTINUE
C*----------------------------------------------------------------------
C*         END BREAKING                                                *
C*----------------------------------------------------------------------
         H(J)=SQRT(ABS(WAX(2,J)*SIGMA(J)/UCGX(2,J)))
         WAY(2,J)=WAYFIX
   63   CONTINUE 
   70   CONTINUE
C*----------------------------------------------------------------------
C*                                                                     *
C*      WRITE OUT RESULTS OF THE CALCULATIONS                          *
C*                                                                     *
C*----------------------------------------------------------------------
        WRITE(IUN3,*)  '   '
        WRITE(IUN3,*)' THETA VALUES:'
        WRITE(IUN3,*)  '   '     
        WRITE(IUN3,14) (TH(JJ)*CONVD*JS,JJ=JF,JL,JS)
        WRITE(IUN4,14) (TH(JJ)*CONVD*JS,JJ=JF,JL,JS)
        WRITE(IUN3,*)  '   '
        WRITE(IUN3,*)' WAVE HEIGHTS:'
        WRITE(IUN3,*)  '   '
        WRITE(IUN3,14) (H(JJ),JJ=JF,JL,JS)
        WRITE(IUN2,14) (H(JJ),JJ=JF,JL,JS)
        WRITE(IUN3,*)  '   '
        WRITE(IUN3,*)' WATER DEPTHS:'
        WRITE(IUN3,*)  '   '
        WRITE(IUN3,14) (D(I1,JJ),JJ=JF,JL,JS)
        WRITE(IUN3,*)  '   '
C*----------------------------------------------------------------------
C*                                                                     *
C*    ROLL DOWN THE VALUES OF A,B                                      *
C*                                                                     *
C*----------------------------------------------------------------------
        DO 490 J=1,N
          A(1,J)=A(2,J)
          B(1,J)=B(2,J)
          K(1,J)=K(2,J)
          UCGX(1,J)=UCGX(2,J)
          VCGY(1,J)=VCGY(2,J)
          WAX(1,J)=WAX(2,J)
          WAY(1,J)=WAY(2,J) 
  490   CONTINUE
  500 CONTINUE 
      CLOSE(IUN2)
      CLOSE(IUN4)       
      STOP 
      END
C***********************************************************************
C*                                                                     *
      SUBROUTINE WVNUM(D,TH,H,I,J,U,V,RK,ITYPE,RKS,G)
C*                                                                     *
C*    SUBROUTINE WVNUM COMPUTES WAVE NUMBER INCLUDING WAVE-            *
C*    CURRENT INTERACTION, USING SECANT METHOD.                        *
C*                                                                     *
C*    OMEGA = SIGMA + U dot K                                          *
C*                                                                     *
C*    NOTE:  ITYPE DENOTES TYPE OF SOLUTION                            *
C*           ITYPE=2, NORMAL SOLUTION                                  *
C*           ITYPE=1, SNELLS LAW                                       *
C***********************************************************************
      PARAMETER(IY=85)
      COMMON /FREQ/OMEGA,SIGMA(IY),CG(IY),NONLIN
      EPSK=0.000001
      AMP=H/2.
      IF(D.LT..01) THEN
      D=.01
      WRITE(IUN3,*) ' DRY LAND AT', J
      ENDIF
      RK=OMEGA*OMEGA/(G*SQRT(TANH(OMEGA*OMEGA*D/G)))
      COSI=COS(TH)
      SINI=SIN(TH)
      BETA=U*COSI+V*SINI
      DO 100 ITER=1,50
      IF(ITYPE.EQ.1)BETA=U*COSI+V*RKS/RK
      SIGMA(J)=OMEGA-RK*BETA
      SIG=SIGMA(J)
      A2=SIG**2
      ARG=RK*D
      TT=TANH(ARG)
      TT2=TT*TT
      SECH2=1.-TT*TT
      EP=RK*AMP
      E=EXP(-2.*ARG)
      E2=E*E
C*----------------------------------------------------------------------
C*    ADD NONLINEAR TERMS                                            *
C*----------------------------------------------------------------------
      IF (NONLIN.EQ.1) THEN
      DD=(9.-TT2*(12.+TT2*(13.-2.*TT2)))/(8.*TT2*TT2) 
      DDK=-D*(1-TT2)*(4.*DD+(6.-TT2*(13.-3.*TT2))/(2.*TT2))/TT    
      F1=TT**5
      F1K=5.*F1*D*SECH2/TT
      F2=16.*(ARG**4)*E2/(1.-E*(4.-E*(6.-E*(4.-E))))
      F2K=4.*F2*(1.-ARG/TT)/RK 
      ELSE 
      DD=0.
      DDK=0.
      F1=0.
      F2=0.
      F1K=0.
      F2K=0.
      ENDIF
      FACT=(1.+F1*EP*EP*DD)
      TAT=TANH(ARG+F2*EP)
C*----------------------------------------------------------------------
C*    USE NEWTON-RAPHSON ITERATION                                 *
C*----------------------------------------------------------------------
      FK=A2-G*RK*TAT*FACT
      FFK=-2.*BETA*SIG-G*(FACT+RK*EP*EP*(F1*DDK+F1K*DD))*TAT
     1-G*RK*FACT*(D+EP*F2K)*(1.-TAT*TAT)
      RKNEW=RK-FK/FFK
      IF(ABS(RKNEW-RK).LE.(ABS(EPSK*RKNEW)))THEN
      RK=RKNEW
      SIGMA(J)=OMEGA-RK*BETA
      CG(J)=(SIGMA(J)/(RK*2.))*(1.-EXP(-4.*ARG)+4.*ARG*EXP(-2.*ARG))
     1/(1.-EXP(-4.*ARG))
      GO TO 110
      ENDIF 
   99 RKO=RK
      FKO=FK
      RK=RKNEW
  100 CONTINUE
      WRITE(IUN3,101)RK,D,H
      RETURN
  110 CONTINUE              
      IF(RK.GT.0.0)GO TO 120
      WRITE(IUN3,130)D,COSI,SINI,U,V,I,J
  120 RETURN
  101 FORMAT(' INTERATION FOR K FAILED TO CONVERGE: 
     -K,D,H,METH',3F10.5,I3)
  130 FORMAT(' RK IS NEGATIVE: D,COSI,SINI,U,V,I,J',
     *5F10.5,2I5)
      END                                                              




